/*
 * text_renderer.h
 *
 * Copyright (C) 2011 Tim Marston <tim@ed.am> and Dan Marston.
 *
 * This file is part of propeller-clock (hereafter referred to as "this
 * program"). See http://ed.am/dev/software/arduino/propeller-clock for more
 * information.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _TEXT_RENDERER_H_
#define _TEXT_RENDERER_H_


#include <PString.h>


#define MAX_MESSAGE_LENGTH 64


class TextRenderer
{
public:

	/**
	 * Get singleton instance of text renderer.
	 *
	 * @return singleton instance
	 */
	static TextRenderer &get_instance();

	/**
	 * Retrieve the PString that is used to render text with the intention of
	 * changing it. This resets the text services.
	 *
	 * @param message the message
	 */
	PString &get_pstring();

	/**
	 * Call at the start of each frame.
	 */
	void draw_reset();

	/**
	 * Draw segment using automatic scrolling.
	 */
	void draw_scroll( int segment );

protected:

	TextRenderer();

	/**
	 * Transform segment in to display-space.
	 *
	 * @param segment
	 * @return display-space segment
	 */
	int xform_segment( int segment );

	/**
	 * Draw the specified column of pixels from the message
	 *
	 * @param x pixel column to draw
	 */
	void render( int x );

	/**
	 * Get a pointer to the character data in the font for a given
	 * character.
	 *
	 * @param character to look-up
	 * @param glyph where to store the glyph data
	 */
	void get_glyph( char c, char *glyph );

	/** the buffer for the message */
	char _message_buffer[ MAX_MESSAGE_LENGTH ];

	/** the current message */
	PString _message;

	/** glyph */
	char _glyph_cache[ 8 ];

	/** the character the current glyph is for */
	char _glyph_cache_char;

	/** selected font */
	int _font;

	/** time last frame */
	unsigned long _last_millis;

	/** duration last frame (in case we can't calculate it this
	 * frame!) */
	unsigned long _duration;

	/** scroll position (in segments ) */
	signed long _scroll;

};


#endif //_TEXT_RENDERER_H_
